@@ -58,9 +58,8 @@ list *listAddNodeHead(list *list, void *value);
 list *listAddNodeTail(list *list, void *value);
 list *listInsertNode(list *list, listNode *old_node, void *value, int after);
 void listDelNode(list *list, listNode *node);
-listIter *listGetIterator(list *list, int direction);
+void listInitIterator(listIter *iter, list *list, int direction);
 listNode *listNext(listIter *iter);
-void listReleaseIterator(listIter *iter);
 list *listDup(list *orig);
 listNode *listSearchKey(list *list, void *key);
 listNode *listIndex(list *list, long index);
