@@ -76,8 +76,9 @@ static void maybeConvertToIntset(robj *set) {
     char *str;
     size_t len = 0;
     int64_t llval = 0;
-    setTypeIterator *si = setTypeInitIterator(set);
-    while (setTypeNext(si, &str, &len, &llval) != -1) {
+    setTypeIterator si;
+    setTypeInitIterator(&si, set);
+    while (setTypeNext(&si, &str, &len, &llval) != -1) {
         if (str) {
             /* If the element is returned as a string, we may be able to convert
              * it to integer. This happens for OBJ_ENCODING_HT. */
@@ -87,7 +88,7 @@ static void maybeConvertToIntset(robj *set) {
         is = intsetAdd(is, llval, &success);
         serverAssert(success);
     }
-    setTypeReleaseIterator(si);
+    setTypeResetIterator(&si);
     freeSetObject(set); /* frees the internals but not robj itself */
     set->ptr = is;
     set->encoding = OBJ_ENCODING_INTSET;
@@ -313,12 +314,11 @@ int setTypeIsMemberAux(robj *set, char *str, size_t len, int64_t llval, int str_
     }
 }
 
-setTypeIterator *setTypeInitIterator(robj *subject) {
-    setTypeIterator *si = zmalloc(sizeof(setTypeIterator));
+void setTypeInitIterator(setTypeIterator *si, robj *subject) {
     si->subject = subject;
     si->encoding = subject->encoding;
     if (si->encoding == OBJ_ENCODING_HT) {
-        si->di = dictGetIterator(subject->ptr);
+        dictInitIterator(&si->di, subject->ptr);
     } else if (si->encoding == OBJ_ENCODING_INTSET) {
         si->ii = 0;
     } else if (si->encoding == OBJ_ENCODING_LISTPACK) {
@@ -326,13 +326,11 @@ setTypeIterator *setTypeInitIterator(robj *subject) {
     } else {
         serverPanic("Unknown set encoding");
     }
-    return si;
 }
 
-void setTypeReleaseIterator(setTypeIterator *si) {
+void setTypeResetIterator(setTypeIterator *si) {
     if (si->encoding == OBJ_ENCODING_HT)
-        dictReleaseIterator(si->di);
-    zfree(si);
+        dictResetIterator(&si->di);
 }
 
 /* Move to the next entry in the set. Returns the object at the current
@@ -358,7 +356,7 @@ void setTypeReleaseIterator(setTypeIterator *si) {
  * When there are no more elements -1 is returned. */
 int setTypeNext(setTypeIterator *si, char **str, size_t *len, int64_t *llele) {
     if (si->encoding == OBJ_ENCODING_HT) {
-        dictEntry *de = dictNext(si->di);
+        dictEntry *de = dictNext(&si->di);
         if (de == NULL) return -1;
         *str = dictGetKey(de);
         *len = sdslen(*str);
@@ -507,7 +505,7 @@ void setTypeConvert(robj *setobj, int enc) {
  * C_ERR on OOM (panic=0). If panic=1 is given, this function always returns
  * C_OK. */
 int setTypeConvertAndExpand(robj *setobj, int enc, unsigned long cap, int panic) {
-    setTypeIterator *si;
+    setTypeIterator si;
     serverAssertWithInfo(NULL,setobj,setobj->type == OBJ_SET &&
                              setobj->encoding != enc);
 
@@ -525,12 +523,12 @@ int setTypeConvertAndExpand(robj *setobj, int enc, unsigned long cap, int panic)
 
         /* To add the elements we extract integers and create redis objects */
         size_t *alloc_size = htGetMetadataSize(d);
-        si = setTypeInitIterator(setobj);
-        while ((element = setTypeNextObject(si)) != NULL) {
+        setTypeInitIterator(&si, setobj);
+        while ((element = setTypeNextObject(&si)) != NULL) {
             serverAssert(dictAdd(d,element,NULL) == DICT_OK);
             *alloc_size += sdsAllocSize(element);
         }
-        setTypeReleaseIterator(si);
+        setTypeResetIterator(&si);
 
         freeSetObject(setobj); /* frees the internals but not setobj itself */
         setobj->encoding = OBJ_ENCODING_HT;
@@ -548,14 +546,14 @@ int setTypeConvertAndExpand(robj *setobj, int enc, unsigned long cap, int panic)
         char *str;
         size_t len = 0;
         int64_t llele = 0;
-        si = setTypeInitIterator(setobj);
-        while (setTypeNext(si, &str, &len, &llele) != -1) {
+        setTypeInitIterator(&si, setobj);
+        while (setTypeNext(&si, &str, &len, &llele) != -1) {
             if (str != NULL)
                 lp = lpAppend(lp, (unsigned char *)str, len);
             else
                 lp = lpAppendInteger(lp, llele);
         }
-        setTypeReleaseIterator(si);
+        setTypeResetIterator(&si);
 
         freeSetObject(setobj); /* frees the internals but not setobj itself */
         setobj->encoding = OBJ_ENCODING_LISTPACK;
@@ -573,7 +571,6 @@ int setTypeConvertAndExpand(robj *setobj, int enc, unsigned long cap, int panic)
  * The resulting object always has refcount set to 1 */
 robj *setTypeDup(robj *o) {
     robj *set;
-    setTypeIterator *si;
 
     serverAssert(o->type == OBJ_SET);
 
@@ -596,14 +593,15 @@ robj *setTypeDup(robj *o) {
         set = createSetObject();
         dict *d = o->ptr;
         dictExpand(set->ptr, dictSize(d));
-        si = setTypeInitIterator(o);
+        setTypeIterator si;
+        setTypeInitIterator(&si, o);
         char *str;
         size_t len = 0;
         int64_t intobj = 0;
-        while (setTypeNext(si, &str, &len, &intobj) != -1) {
+        while (setTypeNext(&si, &str, &len, &intobj) != -1) {
             setTypeAdd(set, (sds)str);
         }
-        setTypeReleaseIterator(si);
+        setTypeResetIterator(&si);
     } else {
         serverPanic("Unknown set encoding");
     }
@@ -623,24 +621,24 @@ void saddCommand(client *c) {
         robj *o = setTypeCreate(c->argv[2]->ptr, c->argc - 2);
         set = dbAddByLink(c->db, c->argv[1], &o, &link);
     } else {
-        if (server.memory_tracking_per_slot)
-            oldsize = setTypeAllocSize(set);
+        if (server.memory_tracking_enabled)
+            oldsize = kvobjAllocSize(set);
         setTypeMaybeConvert(set, c->argc - 2);
-        if (server.memory_tracking_per_slot)
-            updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(set));
+        if (server.memory_tracking_enabled)
+            updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), set, oldsize, kvobjAllocSize(set));
     }
 
-    if (server.memory_tracking_per_slot)
-        oldsize = setTypeAllocSize(set);
+    if (server.memory_tracking_enabled)
+        oldsize = kvobjAllocSize(set);
     for (j = 2; j < c->argc; j++) {
         if (setTypeAdd(set,c->argv[j]->ptr)) added++;
     }
-    if (server.memory_tracking_per_slot)
-        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(set));
+    if (server.memory_tracking_enabled)
+        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), set, oldsize, kvobjAllocSize(set));
     if (added) {
         unsigned long size = setTypeSize(set);
         updateKeysizesHist(c->db, getKeySlot(c->argv[1]->ptr), OBJ_SET, size - added, size);
-        signalModifiedKey(c,c->db,c->argv[1]);
+        keyModified(c,c->db,c->argv[1],set,1);
         notifyKeyspaceEvent(NOTIFY_SET,"sadd",c->argv[1],c->db->id);
     }
     server.dirty += added;
@@ -656,27 +654,27 @@ void sremCommand(client *c) {
         return;
 
     unsigned long oldSize = setTypeSize(set);
-    if (server.memory_tracking_per_slot)
-        oldsize = setTypeAllocSize(set);
+    if (server.memory_tracking_enabled)
+        oldsize = kvobjAllocSize(set);
 
     for (j = 2; j < c->argc; j++) {
         if (setTypeRemove(set,c->argv[j]->ptr)) {
             deleted++;
             if (setTypeSize(set) == 0) {
-                if (server.memory_tracking_per_slot)
-                    updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(set));
+                if (server.memory_tracking_enabled)
+                    updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), set, oldsize, kvobjAllocSize(set));
                 dbDeleteSkipKeysizesUpdate(c->db, c->argv[1]);
                 keyremoved = 1;
                 break;
             }
         }
     }
-    if (server.memory_tracking_per_slot && !keyremoved)
-        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(set));
+    if (server.memory_tracking_enabled && !keyremoved)
+        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), set, oldsize, kvobjAllocSize(set));
     if (deleted) {
         int64_t newSize = oldSize - deleted;
 
-        signalModifiedKey(c,c->db,c->argv[1]);
+        keyModified(c, c->db, c->argv[1], keyremoved ? NULL : set, 1);
         notifyKeyspaceEvent(NOTIFY_SET,"srem",c->argv[1],c->db->id);
         if (keyremoved) {
             notifyKeyspaceEvent(NOTIFY_GENERIC,"del",c->argv[1],
@@ -714,11 +712,11 @@ void smoveCommand(client *c) {
         return;
     }
 
-    if (server.memory_tracking_per_slot)
-        oldSrcAllocSize = setTypeAllocSize(srcset);
+    if (server.memory_tracking_enabled)
+        oldSrcAllocSize = kvobjAllocSize(srcset);
     int deleted = setTypeRemove(srcset,ele->ptr);
-    if (server.memory_tracking_per_slot)
-        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldSrcAllocSize, setTypeAllocSize(srcset));
+    if (server.memory_tracking_enabled)
+        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), srcset, oldSrcAllocSize, kvobjAllocSize(srcset));
     /* If the element cannot be removed from the src set, return 0. */
     if (!deleted) {
         addReply(c,shared.czero);
@@ -743,21 +741,21 @@ void smoveCommand(client *c) {
         dbAdd(c->db, c->argv[2], &dstset);
     }
 
-    signalModifiedKey(c,c->db,c->argv[1]);
+    keyModified(c, c->db, c->argv[1], (srcNewLen > 0) ? srcset : NULL, 1);
     server.dirty++;
 
-    if (server.memory_tracking_per_slot)
-        oldDstAllocSize = setTypeAllocSize(dstset);
+    if (server.memory_tracking_enabled)
+        oldDstAllocSize = kvobjAllocSize(dstset);
     /* An extra key has changed when ele was successfully added to dstset */
     if (setTypeAdd(dstset,ele->ptr)) {
         unsigned long dstLen = setTypeSize(dstset);
         updateKeysizesHist(c->db, getKeySlot(c->argv[2]->ptr), OBJ_SET, dstLen - 1, dstLen);
         server.dirty++;
-        signalModifiedKey(c,c->db,c->argv[2]);
+        keyModified(c,c->db,c->argv[2],dstset,1);
         notifyKeyspaceEvent(NOTIFY_SET,"sadd",c->argv[2],c->db->id);
     }
-    if (server.memory_tracking_per_slot)
-        updateSlotAllocSize(c->db, getKeySlot(c->argv[2]->ptr), oldDstAllocSize, setTypeAllocSize(dstset));
+    if (server.memory_tracking_enabled)
+        updateSlotAllocSize(c->db, getKeySlot(c->argv[2]->ptr), dstset, oldDstAllocSize, kvobjAllocSize(dstset));
     addReply(c,shared.cone);
 }
 
@@ -768,14 +766,14 @@ void sismemberCommand(client *c) {
     if ((set = lookupKeyReadOrReply(c,c->argv[1],shared.czero)) == NULL ||
         checkType(c,set,OBJ_SET)) return;
 
-    if (server.memory_tracking_per_slot)
-        oldsize = setTypeAllocSize(set);
+    if (server.memory_tracking_enabled)
+        oldsize = kvobjAllocSize(set);
     if (setTypeIsMember(set,c->argv[2]->ptr))
         addReply(c,shared.cone);
     else
         addReply(c,shared.czero);
-    if (server.memory_tracking_per_slot)
-        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(set));
+    if (server.memory_tracking_enabled)
+        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), set, oldsize, kvobjAllocSize(set));
 }
 
 void smismemberCommand(client *c) {
@@ -787,16 +785,16 @@ void smismemberCommand(client *c) {
 
     addReplyArrayLen(c,c->argc - 2);
 
-    if (server.memory_tracking_per_slot && set)
-        setTypeAllocSize(set);
+    if (server.memory_tracking_enabled && set)
+        oldsize = kvobjAllocSize(set);
     for (int j = 2; j < c->argc; j++) {
         if (set && setTypeIsMember(set,c->argv[j]->ptr))
             addReply(c,shared.cone);
         else
             addReply(c,shared.czero);
     }
-    if (server.memory_tracking_per_slot && set)
-        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(set));
+    if (server.memory_tracking_enabled && set)
+        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), set, oldsize, kvobjAllocSize(set));
 }
 
 void scardCommand(client *c) {
@@ -860,7 +858,7 @@ void spopWithCountCommand(client *c) {
         /* Propagate this command as a DEL or UNLINK operation */
         robj *aux = server.lazyfree_lazy_server_del ? shared.unlink : shared.del;
         rewriteClientCommandVector(c, 2, aux, c->argv[1]);
-        signalModifiedKey(c,c->db,c->argv[1]);
+        keyModified(c,c->db,c->argv[1],NULL,1);
         return;
     }
 
@@ -891,8 +889,8 @@ void spopWithCountCommand(client *c) {
         set->encoding == OBJ_ENCODING_LISTPACK)
     {
         /* Specialized case for listpack. Traverse it only once. */
-        if (server.memory_tracking_per_slot)
-            oldsize = setTypeAllocSize(set);
+        if (server.memory_tracking_enabled)
+            oldsize = kvobjAllocSize(set);
         unsigned char *lp = set->ptr;
         unsigned char *p = lpFirst(lp);
         unsigned int index = 0;
@@ -927,11 +925,11 @@ void spopWithCountCommand(client *c) {
         zfree(ps);
         set->ptr = lp;
         updateKeysizesHist(c->db, getKeySlot(c->argv[1]->ptr), OBJ_SET, size, size - count);
-        if (server.memory_tracking_per_slot)
-            updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(set));
+        if (server.memory_tracking_enabled)
+            updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), set, oldsize, kvobjAllocSize(set));
     } else if (remaining*SPOP_MOVE_STRATEGY_MUL > count) {
-        if (server.memory_tracking_per_slot)
-            oldsize = setTypeAllocSize(set);
+        if (server.memory_tracking_enabled)
+            oldsize = kvobjAllocSize(set);
         for (unsigned long i = 0; i < count; i++) {
             propargv[propindex] = setTypePopRandom(set);
             addReplyBulk(c, propargv[propindex]);
@@ -946,8 +944,8 @@ void spopWithCountCommand(client *c) {
             }
         }
         updateKeysizesHist(c->db, getKeySlot(c->argv[1]->ptr), OBJ_SET, size, size - count);
-        if (server.memory_tracking_per_slot)
-            updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(set));
+        if (server.memory_tracking_enabled)
+            updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), set, oldsize, kvobjAllocSize(set));
     } else {
     /* CASE 3: The number of elements to return is very big, approaching
      * the size of the set itself. After some time extracting random elements
@@ -958,8 +956,8 @@ void spopWithCountCommand(client *c) {
      * set). Then we return the elements left in the original set and
      * release it. */
         robj *newset = NULL;
-        if (server.memory_tracking_per_slot)
-            oldsize = setTypeAllocSize(set);
+        if (server.memory_tracking_enabled)
+            oldsize = kvobjAllocSize(set);
 
         /* Create a new set with just the remaining elements. */
         if (set->encoding == OBJ_ENCODING_LISTPACK) {
@@ -993,9 +991,9 @@ void spopWithCountCommand(client *c) {
         }
 
         /* Transfer the old set to the client. */
-        setTypeIterator *si;
-        si = setTypeInitIterator(set);
-        while (setTypeNext(si, &str, &len, &llele) != -1) {
+        setTypeIterator si;
+        setTypeInitIterator(&si, set);
+        while (setTypeNext(&si, &str, &len, &llele) != -1) {
             if (str == NULL) {
                 addReplyBulkLongLong(c,llele);
                 propargv[propindex++] = createStringObjectFromLongLong(llele);
@@ -1012,16 +1010,17 @@ void spopWithCountCommand(client *c) {
                 propindex = 2;
             }
         }
-        setTypeReleaseIterator(si);
+        setTypeResetIterator(&si);
 
         /* Update key size histogram "explicitly" and not indirectly by dbReplaceValue()
          * since function dbReplaceValue() assumes the entire set is being replaced, 
          * but here we're building the new set from the existing one. As a result, 
          * the size of the old set has already changed by the time we reach this point. */
         updateKeysizesHist(c->db, getKeySlot(c->argv[1]->ptr), OBJ_SET, size, size-count);
-        if (server.memory_tracking_per_slot)
-            updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(set));
+        if (server.memory_tracking_enabled)
+            updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), set, oldsize, kvobjAllocSize(set));
         dbReplaceValue(c->db, c->argv[1], &newset, 0);
+        set = newset;
     }
 
     /* Replicate/AOF the remaining elements as an SREM operation */
@@ -1039,7 +1038,7 @@ void spopWithCountCommand(client *c) {
      * we propagated the command as a set of SREMs operations using
      * the alsoPropagate() API. */
     preventCommandPropagation(c);
-    signalModifiedKey(c,c->db,c->argv[1]);
+    keyModified(c,c->db,c->argv[1],set,1);
 }
 
 void spopCommand(client *c) {
@@ -1063,14 +1062,14 @@ void spopCommand(client *c) {
     size = setTypeSize(kv);
     updateKeysizesHist(c->db, getKeySlot(c->argv[1]->ptr), OBJ_SET, size, size-1);
 
-    if (server.memory_tracking_per_slot)
-        oldsize = setTypeAllocSize(kv);
+    if (server.memory_tracking_enabled)
+        oldsize = kvobjAllocSize(kv);
 
     /* Pop a random element from the kv */
     ele = setTypePopRandom(kv);
 
-    if (server.memory_tracking_per_slot)
-        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(kv));
+    if (server.memory_tracking_enabled)
+        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), kv, oldsize, kvobjAllocSize(kv));
 
     notifyKeyspaceEvent(NOTIFY_SET,"spop",c->argv[1],c->db->id);
 
@@ -1082,13 +1081,15 @@ void spopCommand(client *c) {
     decrRefCount(ele);
 
     /* Delete the kv if it's empty */
+    int deleted = 0;
     if (setTypeSize(kv) == 0) {
+        deleted = 1;
         dbDelete(c->db,c->argv[1]);
         notifyKeyspaceEvent(NOTIFY_GENERIC,"del",c->argv[1],c->db->id);
     }
 
     /* Set has been modified */
-    signalModifiedKey(c,c->db,c->argv[1]);
+    keyModified(c, c->db, c->argv[1], deleted ? NULL : kv, 1);
     server.dirty++;
 }
 
@@ -1183,10 +1184,10 @@ void srandmemberWithCountCommand(client *c) {
      * The number of requested elements is greater than the number of
      * elements inside the set: simply return the whole set. */
     if (count >= size) {
-        setTypeIterator *si;
+        setTypeIterator si;
         addReplyArrayLen(c,size);
-        si = setTypeInitIterator(set);
-        while (setTypeNext(si, &str, &len, &llele) != -1) {
+        setTypeInitIterator(&si, set);
+        while (setTypeNext(&si, &str, &len, &llele) != -1) {
             if (str == NULL) {
                 addReplyBulkLongLong(c,llele);
             } else {
@@ -1194,7 +1195,7 @@ void srandmemberWithCountCommand(client *c) {
             }
             size--;
         }
-        setTypeReleaseIterator(si);
+        setTypeResetIterator(&si);
         serverAssert(size==0);
         return;
     }
@@ -1240,12 +1241,12 @@ void srandmemberWithCountCommand(client *c) {
      * a bit less than the number of elements in the set, the natural approach
      * used into CASE 4 is highly inefficient. */
     if (count*SRANDMEMBER_SUB_STRATEGY_MUL > size) {
-        setTypeIterator *si;
+        setTypeIterator si;
 
         /* Add all the elements into the temporary dictionary. */
-        si = setTypeInitIterator(set);
+        setTypeInitIterator(&si, set);
         dictExpand(d, size);
-        while (setTypeNext(si, &str, &len, &llele) != -1) {
+        while (setTypeNext(&si, &str, &len, &llele) != -1) {
             int retval = DICT_ERR;
 
             if (str == NULL) {
@@ -1255,7 +1256,7 @@ void srandmemberWithCountCommand(client *c) {
             }
             serverAssert(retval == DICT_OK);
         }
-        setTypeReleaseIterator(si);
+        setTypeResetIterator(&si);
         serverAssert(dictSize(d) == size);
 
         /* Remove random elements to reach the right count. */
@@ -1329,11 +1330,11 @@ void srandmemberCommand(client *c) {
     if ((set = lookupKeyReadOrReply(c,c->argv[1],shared.null[c->resp]))
         == NULL || checkType(c,set,OBJ_SET)) return;
 
-    if (server.memory_tracking_per_slot)
-        oldsize = setTypeAllocSize(set);
+    if (server.memory_tracking_enabled)
+        oldsize = kvobjAllocSize(set);
     setTypeRandomElement(set, &str, &len, &llele);
-    if (server.memory_tracking_per_slot)
-        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(set));
+    if (server.memory_tracking_enabled)
+        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), set, oldsize, kvobjAllocSize(set));
     if (str == NULL) {
         addReplyBulkLongLong(c,llele);
     } else {
@@ -1377,7 +1378,7 @@ void sinterGenericCommand(client *c, robj **setkeys,
                           unsigned long setnum, robj *dstkey,
                           int cardinality_only, unsigned long limit) {
     setopsrc *sets = zmalloc(sizeof(setopsrc)*setnum);
-    setTypeIterator *si;
+    setTypeIterator si;
     robj *dstset = NULL;
     char *str;
     size_t len = 0;
@@ -1400,8 +1401,8 @@ void sinterGenericCommand(client *c, robj **setkeys,
             return;
         }
         sets[j].set = kv;
-        if (server.memory_tracking_per_slot)
-            sets[j].oldsize = setTypeAllocSize(kv);
+        if (server.memory_tracking_enabled)
+            sets[j].oldsize = kvobjAllocSize(kv);
     }
 
     /* Set intersection with an empty set always results in an empty set.
@@ -1410,7 +1411,7 @@ void sinterGenericCommand(client *c, robj **setkeys,
         zfree(sets);
         if (dstkey) {
             if (dbDelete(c->db,dstkey)) {
-                signalModifiedKey(c,c->db,dstkey);
+                keyModified(c,c->db,dstkey,NULL,1);
                 notifyKeyspaceEvent(NOTIFY_GENERIC,"del",dstkey,c->db->id);
                 server.dirty++;
             }
@@ -1461,8 +1462,8 @@ void sinterGenericCommand(client *c, robj **setkeys,
      * the element against all the other sets, if at least one set does
      * not include the element it is discarded */
     int only_integers = 1;
-    si = setTypeInitIterator(sets[0].set);
-    while((encoding = setTypeNext(si, &str, &len, &intobj)) != -1) {
+    setTypeInitIterator(&si, sets[0].set);
+    while((encoding = setTypeNext(&si, &str, &len, &intobj)) != -1) {
         for (j = 1; j < setnum; j++) {
             if (sets[j].set == sets[0].set) continue;
             if (!setTypeIsMemberAux(sets[j].set, str, len, intobj,
@@ -1505,13 +1506,14 @@ void sinterGenericCommand(client *c, robj **setkeys,
             }
         }
     }
-    setTypeReleaseIterator(si);
+    setTypeResetIterator(&si);
 
-    if (server.memory_tracking_per_slot) {
+    if (server.memory_tracking_enabled) {
         for (j = 0; j < setnum; j++) {
-            if (!sets[j].set) continue;
-            updateSlotAllocSize(c->db, getKeySlot(setkeys[j]->ptr),
-                            sets[j].oldsize, setTypeAllocSize(sets[j].set));
+            robj *obj = sets[j].set;
+            if (!obj) continue;
+            updateSlotAllocSize(c->db, getKeySlot(setkeys[j]->ptr), obj,
+                                sets[j].oldsize, kvobjAllocSize(obj));
         }
     }
 
@@ -1536,7 +1538,7 @@ void sinterGenericCommand(client *c, robj **setkeys,
             addReply(c,shared.czero);
             if (dbDelete(c->db,dstkey)) {
                 server.dirty++;
-                signalModifiedKey(c,c->db,dstkey);
+                keyModified(c,c->db,dstkey,NULL,1);
                 notifyKeyspaceEvent(NOTIFY_GENERIC,"del",dstkey,c->db->id);
             }
             decrRefCount(dstset);
@@ -1554,7 +1556,7 @@ void sinterCommand(client *c) {
 
 /* SMEMBERS key */
 void smembersCommand(client *c) {
-    setTypeIterator *si;
+    setTypeIterator si;
     char *str;
     size_t len = 0;
     int64_t intobj = 0;
@@ -1569,21 +1571,21 @@ void smembersCommand(client *c) {
     /* Prepare the response. */
     unsigned long length = setTypeSize(setobj);
     addReplySetLen(c,length);
-    if (server.memory_tracking_per_slot)
-        oldsize = setTypeAllocSize(setobj);
+    if (server.memory_tracking_enabled)
+        oldsize = kvobjAllocSize(setobj);
     /* Iterate through the elements of the set. */
-    si = setTypeInitIterator(setobj);
+    setTypeInitIterator(&si, setobj);
 
-    while (setTypeNext(si, &str, &len, &intobj) != -1) {
+    while (setTypeNext(&si, &str, &len, &intobj) != -1) {
         if (str != NULL)
             addReplyBulkCBuffer(c, str, len);
         else
             addReplyBulkLongLong(c, intobj);
         length--;
     }
-    setTypeReleaseIterator(si);
-    if (server.memory_tracking_per_slot)
-        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(setobj));
+    setTypeResetIterator(&si);
+    if (server.memory_tracking_enabled)
+        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), setobj, oldsize, kvobjAllocSize(setobj));
     serverAssert(length == 0); /* fail on corrupt data */
 }
 
@@ -1627,7 +1629,7 @@ void sinterstoreCommand(client *c) {
 void sunionDiffGenericCommand(client *c, robj **setkeys, int setnum,
                               robj *dstkey, int op) {
     setopsrc *sets = zmalloc(sizeof(setopsrc)*setnum);
-    setTypeIterator *si;
+    setTypeIterator si;
     robj *dstset = NULL;
     int dstset_encoding = OBJ_ENCODING_INTSET;
     char *str;
@@ -1667,8 +1669,8 @@ void sunionDiffGenericCommand(client *c, robj **setkeys, int setnum,
             dstset_encoding = OBJ_ENCODING_HT;
         }
         sets[j].set = setobj;
-        if (server.memory_tracking_per_slot)
-            sets[j].oldsize = setTypeAllocSize(setobj);
+        if (server.memory_tracking_enabled)
+            sets[j].oldsize = kvobjAllocSize(setobj);
         if (j > 0 && sets[0].set == sets[j].set) {
             sameset = 1; 
         }
@@ -1722,11 +1724,11 @@ void sunionDiffGenericCommand(client *c, robj **setkeys, int setnum,
         for (j = 0; j < setnum; j++) {
             if (!sets[j].set) continue; /* non existing keys are like empty sets */
 
-            si = setTypeInitIterator(sets[j].set);
-            while ((encoding = setTypeNext(si, &str, &len, &llval)) != -1) {
+            setTypeInitIterator(&si, sets[j].set);
+            while ((encoding = setTypeNext(&si, &str, &len, &llval)) != -1) {
                 cardinality += setTypeAddAux(dstset, str, len, llval, encoding == OBJ_ENCODING_HT);
             }
-            setTypeReleaseIterator(si);
+            setTypeResetIterator(&si);
         }
     } else if (op == SET_OP_DIFF && sameset) {
         /* At least one of the sets is the same one (same key) as the first one, result must be empty. */
@@ -1739,8 +1741,8 @@ void sunionDiffGenericCommand(client *c, robj **setkeys, int setnum,
          *
          * This way we perform at max N*M operations, where N is the size of
          * the first set, and M the number of sets. */
-        si = setTypeInitIterator(sets[0].set);
-        while ((encoding = setTypeNext(si, &str, &len, &llval)) != -1) {
+        setTypeInitIterator(&si, sets[0].set);
+        while ((encoding = setTypeNext(&si, &str, &len, &llval)) != -1) {
             for (j = 1; j < setnum; j++) {
                 if (!sets[j].set) continue; /* no key is an empty set. */
                 if (sets[j].set == sets[0].set) break; /* same set! */
@@ -1753,7 +1755,7 @@ void sunionDiffGenericCommand(client *c, robj **setkeys, int setnum,
                 cardinality += setTypeAddAux(dstset, str, len, llval, encoding == OBJ_ENCODING_HT);
             }
         }
-        setTypeReleaseIterator(si);
+        setTypeResetIterator(&si);
     } else if (op == SET_OP_DIFF && sets[0].set && diff_algo == 2) {
         /* DIFF Algorithm 2:
          *
@@ -1765,8 +1767,8 @@ void sunionDiffGenericCommand(client *c, robj **setkeys, int setnum,
         for (j = 0; j < setnum; j++) {
             if (!sets[j].set) continue; /* non existing keys are like empty sets */
 
-            si = setTypeInitIterator(sets[j].set);
-            while((encoding = setTypeNext(si, &str, &len, &llval)) != -1) {
+            setTypeInitIterator(&si, sets[j].set);
+            while((encoding = setTypeNext(&si, &str, &len, &llval)) != -1) {
                 if (j == 0) {
                     cardinality += setTypeAddAux(dstset, str, len, llval,
                                                  encoding == OBJ_ENCODING_HT);
@@ -1775,32 +1777,33 @@ void sunionDiffGenericCommand(client *c, robj **setkeys, int setnum,
                                                     encoding == OBJ_ENCODING_HT);
                 }
             }
-            setTypeReleaseIterator(si);
+            setTypeResetIterator(&si);
 
             /* Exit if result set is empty as any additional removal
              * of elements will have no effect. */
             if (cardinality == 0) break;
         }
     }
-    if (server.memory_tracking_per_slot) {
+    if (server.memory_tracking_enabled) {
         for (j = 0; j < setnum; j++) {
-            if (!sets[j].set) continue;
-            updateSlotAllocSize(c->db, getKeySlot(setkeys[j]->ptr),
-                            sets[j].oldsize, setTypeAllocSize(sets[j].set));
+            robj *obj = sets[j].set;
+            if (!obj) continue;
+            updateSlotAllocSize(c->db, getKeySlot(setkeys[j]->ptr), obj,
+                                sets[j].oldsize, kvobjAllocSize(obj));
         }
     }
 
     /* Output the content of the resulting set, if not in STORE mode */
     if (!dstkey) {
         addReplySetLen(c,cardinality);
-        si = setTypeInitIterator(dstset);
-        while (setTypeNext(si, &str, &len, &llval) != -1) {
+        setTypeInitIterator(&si, dstset);
+        while (setTypeNext(&si, &str, &len, &llval) != -1) {
             if (str)
                 addReplyBulkCBuffer(c, str, len);
             else
                 addReplyBulkLongLong(c, llval);
         }
-        setTypeReleaseIterator(si);
+        setTypeResetIterator(&si);
         server.lazyfree_lazy_server_del ? freeObjAsync(NULL, dstset, -1) :
                                           decrRefCount(dstset);
     } else {
@@ -1817,7 +1820,7 @@ void sunionDiffGenericCommand(client *c, robj **setkeys, int setnum,
             addReply(c,shared.czero);
             if (dbDelete(c->db,dstkey)) {
                 server.dirty++;
-                signalModifiedKey(c,c->db,dstkey);
+                keyModified(c,c->db,dstkey,NULL,1);
                 notifyKeyspaceEvent(NOTIFY_GENERIC,"del",dstkey,c->db->id);
             }
             decrRefCount(dstset);
@@ -1854,9 +1857,9 @@ void sscanCommand(client *c) {
     if (parseScanCursorOrReply(c,c->argv[2],&cursor) == C_ERR) return;
     if ((set = lookupKeyReadOrReply(c,c->argv[1],shared.emptyscan)) == NULL ||
         checkType(c,set,OBJ_SET)) return;
-    if (server.memory_tracking_per_slot)
-        oldsize = setTypeAllocSize(set);
+    if (server.memory_tracking_enabled)
+        oldsize = kvobjAllocSize(set);
     scanGenericCommand(c,set,cursor);
-    if (server.memory_tracking_per_slot)
-        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), oldsize, setTypeAllocSize(set));
+    if (server.memory_tracking_enabled)
+        updateSlotAllocSize(c->db, getKeySlot(c->argv[1]->ptr), set, oldsize, kvobjAllocSize(set));
 }
