@@ -1,4 +1,4 @@
-start_server {tags {"repl external:skip"}} {
+start_server {tags {"repl external:skip debug_defrag:skip"}} {
     start_server {} {
         test {First server should have role slave after SLAVEOF} {
             r -1 slaveof [srv 0 host] [srv 0 port]
