@@ -84,6 +84,50 @@ int RedisModule_OnLoad(RedisModuleCtx *ctx, RedisModuleString **argv, int argc)
                     {0}
                 }
             },
+            {
+                .name = "idmp",
+                .type = REDISMODULE_ARG_TYPE_ONEOF,
+                .flags = REDISMODULE_CMD_ARG_OPTIONAL,
+                .subargs = (RedisModuleCommandArg[]){
+                    {
+                        .name = "idmpauto-with-pid",
+                        .type = REDISMODULE_ARG_TYPE_BLOCK,
+                        .subargs = (RedisModuleCommandArg[]){
+                            {
+                                .name = "idmpauto-token",
+                                .type = REDISMODULE_ARG_TYPE_PURE_TOKEN,
+                                .token = "IDMPAUTO"
+                            },
+                            {
+                                .name = "pid",
+                                .type = REDISMODULE_ARG_TYPE_STRING,
+                            },
+                            {0}
+                        }
+                    },
+                    {
+                        .name = "idmp-with-pid-iid",
+                        .type = REDISMODULE_ARG_TYPE_BLOCK,
+                        .subargs = (RedisModuleCommandArg[]){
+                            {
+                                .name = "idmp-token",
+                                .type = REDISMODULE_ARG_TYPE_PURE_TOKEN,
+                                .token = "IDMP"
+                            },
+                            {
+                                .name = "pid",
+                                .type = REDISMODULE_ARG_TYPE_STRING,
+                            },
+                            {
+                                .name = "iid",
+                                .type = REDISMODULE_ARG_TYPE_STRING,
+                            },
+                            {0}
+                        }
+                    },
+                    {0}
+                }
+            },
             {
                 .name = "trim",
                 .type = REDISMODULE_ARG_TYPE_BLOCK,
